/***************************************************************************
                          firewidget.cpp  -  description
                             -------------------
    begin                : Thu Jul 19 12:44:21 EST 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpainter.h>
#include <qfile.h>
#include <qtextstream.h>
#include <time.h>
#include <stdlib.h>
#include "firewidget.h"
#include "palette.h"
#include "firewidget.moc"

FireWidget::FireWidget(QWidget *parent, const char *name) : QFrame(parent, name)
{
	newFrame = 0;
	oldFrame = 0;

	setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
  initFrames();

  srand(time(0));

	fireIntensity = 25;
}

FireWidget::~FireWidget()
{
	if(newFrame)
		delete newFrame;

	if(oldFrame)
		delete oldFrame;
}

void FireWidget::initFrames()
{
    if(newFrame)
        delete newFrame;
    if(oldFrame)
        delete oldFrame;

    newFrame = new QImage(width()+2, height()+2, 8);
    ASSERT(newFrame);
    oldFrame = new QImage(width()+2, height()+2, 8);
    ASSERT(oldFrame);

    newFrame->setNumColors(256);
    oldFrame->setNumColors(256);

/*
		QImage palImg("/home/thorsten/firewidget/palette.bmp");

		QFile file("/home/thorsten/firewidget/palette.h");
		file.open(IO_WriteOnly);
		QTextStream t(&file);
*/
    for(int i=0; i<256; i++)
    {
			newFrame->setColor(i, qRgb(firePalette[i][0], firePalette[i][1], firePalette[i][2]));
			oldFrame->setColor(i, qRgb(firePalette[i][0], firePalette[i][1], firePalette[i][2]));
/*
			newFrame->setColor(i, palImg.color(i));
			oldFrame->setColor(i, palImg.color(i));
			t << "{" << qRed(palImg.color(i)) << ", " << qGreen(palImg.color(i)) << ", " << qBlue(palImg.color(i)) <<"},";
*/
    }
//		file.close();
    newFrame->fill(0);
    oldFrame->fill(0);
}

void FireWidget::setFireIntensity(int intensity)
{
	fireIntensity = intensity;

	if(intensity < 0)
		fireIntensity = 0;
	if(intensity > 100)
		fireIntensity = 100;

	fireIntensity /= 2;
	fireIntensity += 3;
}

void FireWidget::setHotSpots(int num)
{
    int i;
		int w;
    uchar *line;

    line = newFrame->scanLine(newFrame->height()-2);
		w = newFrame->width()-2;
    for(i=0; i<num; i++)
        *(line + (rand() % w) + 1) = 255;
}

void FireWidget::renderNextFrame()
{
    int x, y;
    int accum;
    uchar *src, *dest;
    int offset1;
    QImage *swap;
		static bool moreSpots = true;

		if(moreSpots)
			setHotSpots(fireIntensity);
    moreSpots = !moreSpots;

    offset1 = newFrame->bytesPerLine();
    for(y=0; y<newFrame->height()-1; y++)
    {
        src = newFrame->scanLine(y);
        dest = oldFrame->scanLine(y)+1;

        for(x=1; x<newFrame->width()-1; x++, src++, dest++)
        {
            accum = 0;

            accum += *(src+1);
            accum += *(src+offset1);
            accum += *(src+offset1+1);
            accum += *(src+offset1+2);
            accum /= 4;
            *dest = (uchar)accum;
        }
    }

    swap = newFrame;
    newFrame = oldFrame;
    oldFrame = swap;

    repaint(FALSE);
}

void FireWidget::drawContents(QPainter *painter)
{
    painter->drawImage(0, 0, *newFrame, 1, 0, newFrame->width()-2, newFrame->height()-3);
}

void FireWidget::resizeEvent(QResizeEvent *)
{
  initFrames();
}
